;Accu Alarm Clock
;January 7, 2013 (only change was to turn off brown-out detection)(chip relay output
;		failure - symptom was off-of-relay pulse was sent for each touch of plate
;		instead of on/off pulses.)

;Provides up to 30 alarm settings for an AC outlet.
;seconds correction, periodically made, help keep clock accuracy good


;Requires time to be set.  Electronics will ride through short power failures.  

;The clock flashes out Mountain Standard Time
;  throughout the year.
;     24 hour format .... 
;         single green = hours  ---  double green flashes = addition of 10 for hours
;         single yellow flashes = 10 minute segments of minutes
;         single red flashes = 1 minute each
;
;Fuses set NO brownout detection,  crystal input 3-8 MHz 14ck+68ms Clk.Sel=1101
;
;With 1 F capacitor, unit survives better than 10 minute power loss.                         

; R15 seconds counter

; R16 general purpose temporary use register
; R17 general purpose temporary use register

; R24,R25,R26,R27,R28,R29,R30,R31 = scratch

; R18 is flags register
;	R18,0(1) = AM=0  PM=1
;	R!8,1(2) = Relay status
;	R18,2(4) = Request relay toggle
;	R18,3(8) = single flash flag
;	R18,4(16) = 
;	R18,5(32) = Add one second - counter one
;	R18,6(64) = Add one second - counter two
;	R18,7(128) = 

;	r26 = Button Low Byte of total count
;	r27 = Button High Byte of total count


;	EEPROM:
;	;	$68 - $6B = Clock Correction Interval Permanent Store ($68,$6B = SRAM perm store
;		  $6C - $6F = SRAM variable store)
;	;	$16 - $36 =	Alarm time store area



.include "tn2313def.inc"

;*****Initialization


rjmp	START1
rjmp	START1
rjmp	START1
rjmp	START1
rjmp	ClockCntDwnInt

	
START1:			;set up stack pointer and initialize cold start

ldi r16,low(RAMEND)	; Load low byte address of end of RAM into register R16
out	SPL,R16			; Initialize stack pointer to end of internal RAM

ser	r16			;Set to all ones for out port direction as output
out	DDRB,r16	;Set PORTB to output

ldi	r16,3
out	PortB,r16	;LEDs are off (bits 0 & 1 high) All other ports low

ldi	R16,254		;Set bit 0(1) as input ... remaining bits as output
out	DDRD,R16	;execute data direction with bit 0(1) as input

clr	r16
out	PORTD,R16	;execute to set out ports low, and in port 2(4) with
					; no input resistor


ldi		r16,10		;five seconds delay --- waiting for power supply to come up fully
rcall	FlashGLED

rcall	TransferClockCorrectionFromEEPROM	;Clock corrections and XTL divider data from EEPROM
											;  to SRAM
ldi		r16,$07		;EEPROM r16 address
rcall	FetchEEPROM
sts		$67,r17		;put EEPROM data contained in r17 into SRAM $67, Xtl Divider Store

in	r16,TIMSK	;enable 16-bit match to A register
ori	r16,64		
out	TIMSK,r16

in	r16,TCCR1B
ori	r16,11		; divide by 64 on prescaler
out	TCCR1B,r16	;enable reset of counter to zero (bit3) after match detection.
				;and bits 0,1,2 select prescaling source as 64 of clock

ldi	r16,54		;set up quarter second interrupts
out	OCR1AH,r16
lds	r16,$67		;crystal division determined from test runs --- EEPROM saved data in $67
				; Calculated value is 159
out	OCR1AL,r16	;loads the compare register A for timer 1 (.25 second interrupts)

ldi	r16,4
mov	r11,r16			;Put 4 into divide by 4 counter (interrupt area)			

clr	r18				;zero all flags in register r18
clr	r19				;zero all flags in register r19

ldi		r16,$80
sts		$79,r16		;clear all SRAM alarms

InitializeClkTime:	;starts clock at 8:00 a.m. on a Sunday

ldi	r16,1
sts	$63,r16			;put Sunday(1) in as start of day-of-week

ldi	r16,224			;8:00 A.M. low byte
sts	$61,r16		

ldi	r16,1			;8:00 A.M. high byte
sts	$62,R16

mov	r15,r16			;set seconds counter to one

sei					;enable global interrupts


;------------------------------------

MainLoop:

sbis	PIND,0			;skip if touch plate not touched
rjmp	ButtonService

mov		r16,r15			;get seconds value
cpi		r16,0	
brne	ML2

rjmp	DisplayClock00


ML2:

cpi		r16,30
brne	ML3

rjmp	DisplayClock30


ML3:

rjmp	MainLoop


;------------------------------------

ButtonService:		;Decodes button pushes

rcall	ButtonPush	;r30 has LSB of total count

cpi		r30,0		
brne	BServe2

rcall	ToggleRelay
rjmp	BServeEnd2


BServe2:

cpi		r30,4		;Write new alarm times into SRAM
brne	BServe3

rcall	WriteNewAlarmTimes
rjmp	BServeEnd2


Bserve3:				;Add alarm times to those already in SRAM

cpi		r30,5
brne	BServe3a

rcall	AddToAlarmTimes
rjmp	BServeEnd2


Bserve3a:

cpi		r30,6			;Clear all SRAM clock times
brne	BServe4

ldi		r16,$80
sts		$7F,r16			;clear SRAM alarms (put $80 is $7F bookmark store area)

rjmp	BServeEnd2


BServe4:

cpi		r30,7		;Read all alarm times in EEPROM into SRAM
brne	BServe5

rcall	TransferAlarmTimesFromEEPROM	;put into SRAM
rjmp	BServeEnd2


BServe5:

cpi		r30,10		;Place all SRAM alarm times into EEPROM
brne	BServe9a

rcall	TransferAlarmTimesToEEPROM	;put into EEPROM
rjmp	BServeEnd2


BServe9a:

cpi		r30,22		;Add coarse seconds correction for counter #1 (whole seconds/24 hours)
					; but leaving slight slowdown so counter #2 can do fine fractional correction
brne	BServe10

rcall	CounterOneCorrectionData	;four entries required
rjmp	BServeEnd2


BServe10:

cpi		r30,26		;Enter seconds correction for counter #2 (fractional)
					; This one is for fractional seconds not corrected by counter #1
					; --- billions, millions, thousands, hundreds
brne	BServe11

rcall	CounterTwoCorrectionData	;four entries required
rjmp	BServeEnd2


BServe11:			;adjust the crystal divider

cpi		r30,44
brne	BServe12

rcall	AdjustXtlDivider
rjmp	BServeEnd2


BServe12:			;Set Clock

cpi		r30,30
brne	Bserve13

rcall	SetClock
rjmp	BServeEnd2


BServe13:

cpi		r30,46		;Sets zeros in clock correction and 159 as 
					;  low byte divisor for crstal countdown for timing.
brne	BServe14

rcall	ZeroSRAMeeprom
rjmp	BserveEnd2


BServe14:			;Quick Set Entry.  Quarter hour entries starting at 00:15 to 23:59
					;A toggle only -- no turn off once toggled

cpi		r30,50
brne	BServeEnd

rcall	QuarterEntries
rjmp	BserveEnd2


BServeEnd:

rcall	OneSecondRFlash		;One second flash indicates incorrect entry
rjmp	MainLoop


BserveEnd2:

rcall	TwoWags
rjmp	MainLoop


;-----------------------------------------

;	LED Flash Ladder

TwoGreenLEDFlash:

ldi		r16,2
rjmp	GreenLEDFlash


OneGreenLEDFlash:		;flashes Green LED once

ldi		r16,1


GreenLEDFlash:			;flashes Green LED number of times in r16 (slow count flashes)

tst		r16
breq	GreenEnd

cbi		PortB,1			;ON Green LED
rcall	Delay40
sbi		PortB,1			;OFF Green LED
rcall	DelayHalf		; 1/2 second delay
dec		r16
rjmp	GreenLEDFlash


GreenEnd:

ret


;--------------------------------------------

GreenFlashStall:		;flashes green LED until button is pushed

cbi		PortB,1			;ON green LED

rcall	EscapeTenthDelay

sbi		PortB,1			;OFF green LED

sbis	PIND,0			;skip if button is up
ret

rcall	EscapeHalfDelay

rjmp	GreenFlashStall


;--------------------------------------------

DisplayClock00:


DC00aa:

rcall	EscapeLongRedFlash	;it is 00, so use longer red flash

sbis	PIND,0
rjmp	ButtonService

rcall	EscapeLongRedFlash

sbis	PIND,0			;skip if touch plate not touched
rjmp	ButtonService
rjmp	FlashOut24Time


;----------------------------------------

DisplayClock30:

sbrs	r18,5		;skip if r18,5(32) is set  -- add one second flag counter #1
rjmp	DC30b

cbr		r18,32			;clear correction flag for counter #1
inc		r15				;increment second counter by one for 
						;  fractional clock correction 30 seconds in

sbrs	r18,6			;skip if r18,6(64) is set -- add one second flag counter #2
rjmp	DC30b

cbr		r18,64			;clear correction flag for counter #2
inc		r15				;increment second counter by one second
						;  30 seconds into seconds counter


DC30b:

rcall	EscapeLongRedFlash	;use long red flash -- 30 second readout

sbis	PIND,0			;skip if touch plate not touched
rjmp	ButtonService

rjmp	FlashOut24Time


;-------------------------------------

FlashOut24Time:		;Flashes out DOY and TIME in 24 hour format
					; short red flash indicates DOW (1 = Sundday)
					; On hour - long red flash, half hour shorter red flash
					; Hours in Green.  10 min segments in Yellow.   Min in Red

sbis	PIND,0		;skip if touch plate not touched
rjmp	ButtonService

rcall	EscapeHalfDelay
lds		r16,$63					;get day of week from $63 SRAM
rcall	EscapeTimeFlashMinsRed	;flash out day of week

lds		r30,$61
lds		r31,$62		;Get current time

clr		r16


FO24a:

sbis	PIND,0		;skip if touch plate not touched
rjmp	ButtonService

inc		r16			;count hours
sbiw	r30,60		;subtract 60 minutes from total
brcc	FO24a

adiw	r30,60		;restore correcte value
dec		r16			;correct number of hours

rcall	EscapeTimeFlashHourGreen	;flash out hours (double for >9) - r16 has number of flashes

rcall	EscapeHalfDelay

;r30 now has remaining minutes left


FO24a2:

sbis	PIND,0		;skip if button up
rjmp	ButtonService

clr		r16


FO24a2a:

sbis	PIND,0
rjmp	ButtonService

inc		r16
subi	r30,10		;subtract 10 minutes segments from r30
brcc	FO24a2a

ldi		r19,10
add		r30,r19		;restore correct value in r30
dec		r16			;dec r16 to give correct count

rcall	EscapeTimeFlashTensYellow	;flash out tens count - r16 has number of flashes

rcall	EscapeHalfDelay

;final remainder of single minutes count now in r30

mov		r16,r30

rcall	EscapeTimeFlashMinsRed			;flash out minutes

rjmp	MainLoop


;------------------------------

EscapeTimeFlashHourGreen:	;r16 contain number of flashes
							; determines if double flash or single flash is needed
							; double flash indicates 10X in value

clr		r17					;r17 is double flash counter


ETFG1e:

sbis	PIND,0
ret

inc		r17			;increment double flash count by one
subi	r16,10		;subtract any high numbers
brcc	ETFG1e		;go back for another round

ldi		r19,10
add		r16,r19		;restore old count ---- save for single flash outs
dec		r17			;one less to correct r17 count


ETFG1a:				;gives double flashes

rcall	EscapeHalfDelay


ETFG1aa:

sbis	PIND,0
ret

rcall	EscapeHalfDelay		;pause before flashout

tst		r17
breq	ETFG1b

cbi		PortB,1
rcall	Escape40Delay
sbi		PortB,1
rcall	EscapeTenthDelay
sbis	PIND,0			;skip if touch plate not touched
ret
rcall	Escape40Delay
cbi		PortB,1
rcall	Escape40Delay
sbi		PortB,1			;gives double flash

dec		r17
rjmp	ETFG1aa


ETFG1b:					;single flash outs (in r16)

rcall	EscapeHalfDelay


ETFG1bb:

sbis	PIND,0
ret

rcall	EscapeHalfDelay		;pause before flashout

tst		r16
breq	ETFGend

cbi		PortB,1		;on Green LED --- single flash
rcall	Escape40Delay
sbi		PortB,1		;off Green LED
rcall	Escape40Delay

dec		r16
rjmp	ETFG1bb


ETFGend:

ret


;----------------------

EscapeTimeFlashTensYellow:	;flashes out 10 minute segments of hour - one flash per 10 minutes
							;  r16 has number of flashes

ETFY1:

sbis	PIND,0
ret

rcall	EscapeHalfDelay

tst		r16
breq	ETFYend
ldi		r19,25				;25 iterations of red/green fast flashing


ETFY1a:

sbis	PIND,0
rjmp	ETFYend

cbi		PortB,0				;on red led
rcall	Escape1000thDelay
sbi		PortB,0				;off red led
cbi		PortB,1				;on green led
rcall	Escape1000thDelay
sbi		PortB,1				;off green led
dec		r19
brne	ETFY1a

dec		r16
rjmp	ETFY1


ETFYend:

ret


;------------------------------

EscapeTimeFlashMinsRed:		;r16 has number of red flashes for final flash out

sbis	PIND,0
ret

tst		r16
breq	ETFRend

rcall	EscapeHalfDelay

cbi		PortB,0				;on red led
rcall	Escape40Delay
sbi		PortB,0				;off red led

dec		r16
rjmp	EscapeTimeFlashMinsRed


ETFRend:

ret


;-----------------------------

EscapeLongRedFlash:		;used for error warning and time display
						;  50% on time of LED

push	r16

ldi		r16,250


ELR1:

sbis	PIND,0		;skip if touch plate not touched
rjmp	ELRend

cbi		PortB,0		;on RED LED
rcall	Escape1000thDelay
rcall	Escape1000thDelay
sbi		PortB,0		;off LED
rcall	Escape1000thDelay
rcall	Escape1000thDelay
dec		r16
brne	ELR1

ELRend:

pop		r16

ret


;----------------------------------------------

	;INTERRUPT ROUTINE

;--------------------------------------------------------


ClockCntDwnInt: 		;interrupt to update clock counters

push	r14
push	r16
push	r17
push	r24
push	r25
push	r26
push	r27
push	r28
push	r29
push	r30
push	r31

in		r16,SREG	;save flags
sts		$60,r16

DoubleRegisterCorrection:	;This section uses two 4 register clusters to correct the clock.
							;  Both clusters are identical in correction capability.
							;A zero in any cluster will prevent its use.
							;  A non-zero will cause a one second countdown of the 
							;  4 registers.  When zero is reached, the count is reset from
							;  the permanent stora area for that cluster.
							;Maximum resolution is 1 second correction up to every 4,294,967,296 
							;  seconds.


ldi		r26,$68		;start of SRAM permanent store for counter #1 in x register pair
rcall	LoadR28toR31byR26start	;loads counter #1 perm data and checks for zero
rcall	ChkR28toR31forZero		;clears flag if result is zero
brne	DRC1
rjmp	DRC2		;jump on to check on counter #2 (no action needed for counter #1)


DRC1:

ldi		r26,$6C
rcall	LoadR28toR31byR26start	;load r28-r31 with variable counter #1 count

sbiw	r28,1		;decrement count by one
brcc	DRC1a

sbiw	r30,1		;decrement count by one
brcc	DRC1a

sbr		r18,32		;set r18,5(32) counter one add one second flag
ldi		r26,$68
rcall	LoadR28toR31byR26start	;load count from permanent store for counter #1


DRC1a:				;store count away in variable counter #1

ldi		r26,$6C
rcall	StoreR28toR31byR26start


;-------

ldi		r26,$70		;start of SRAM permanent store for counter #2 in x register pair
rcall	LoadR28toR31byR26start	;loads counter #2 perm data and checks for zero
rcall	ChkR28toR31forZero		;clears flag if result is zero
brne	DRC2
rjmp	DRCend		;jump out -- no decrement needed


DRC2:

ldi		r26,$74		
rcall	LoadR28toR31byR26start	;load variable counter #2 count

sbiw	r28,1		;decrement count by one
brcc	DRC2a

sbiw	r30,1		;decrement count by one
brcc	DRC2a

sbr		r18,64		;set r18,6(64) one second add - counter 2
ldi		r26,$70
rcall	LoadR28toR31byR26start	;reset count from permanent store area


DRC2a:				;store count away in variable counter #2

ldi		r26,$74
rcall	StoreR28toR31byR26start


DRCend:				;End


;---------------------------------------------

dec		r11			;decrement 4 counter, r11

brne	CCDIexit

ldi		r16,4		;load r16 with 4 for a four countdown -- Reset 4 count
mov		r11,r16


;---------------------------------------------

CCDIclock1:

inc		r15			;step one count of SECONDS counter
mov		r16,r15

cpi		r16,60		;check for new minute
breq	CCDI1
rjmp	CCDIexit


CCDI1:				;NEW  MINUTE  

clr		r15			;zero r15 -- SECONDS counter


CCDI1Cntr:

lds		r16,$61		;load low minutes byte into R16

inc		r16
sts		$61,r16

brne	CCDI1a		;jump out if (not equal) -- no zero

lds		r16,$62		;high byte minutes
inc		r16			;increment high byte minutes
sts		$62,r16


CCDI1a:

lds		r16,$62		;check high byte if it is 5
cpi		r16,5

breq	CCDI1b

rjmp	CCDIAlarmExit


CCDI1b:			

lds		r16,$61
cpi		r16,160		;1440 minutes reached -- it's now midnight

breq	CCDI1c

rjmp	CCDIAlarmExit


CCDI1c:			;zero high and low bytes - it is midnight

clr		r16
sts		$61,r16		;Zero minute counter for midnight reset
sts		$62,r16

lds		r16,$63		;increment day of week counter
inc		r16
sts		$63,r16

cpi		r16,8

brlo	CCDIAlarmExit


CCDI1d:

ldi		r16,1		;load with 1 (Sunday)
sts		$63,r16


CCDIAlarmExit:

rcall	CheckAllAlarms


CCDIExit:		;Exit this interrupt routine

lds		r16,$60		;SREG temporary store
out		SREG,r16	;restore SREG data

pop		r31
pop		r30
pop		r29
pop		r28
pop		r27
pop		r26
pop		r25
pop		r24
pop		r17
pop		r16
pop		r14

reti


;-----------------------------


CheckAllAlarms:		;check entire list of alarm commands for match

ldi		r26,$80		;set X register, r26,27 at start of alarm store area in SRAM

lds		r16,$7F		;store book mark location into r16
cpi		r16,$80
brne	CAA2

ret


CAA2:

ld		r30,x+		;load r30 with data poinhted by register X, r26,27, SRAM - post incr
ld		r31,x+		;load r31 with data - post increment 


CAA2a:	

lds		r28,$61		;put low byte of present time into r28
lds		r29,$62		;put high byte of present time into r29

cp		r30,r28		;compare low byte
breq	CAA3
rjmp	CAA2b		;go for another round


CAA3:

mov		r27,r31		;temporarily save r31 high byte//DOW information into r27

andi	r31,7		;save lower 3 bytes of command for time comparison

cp		r29,r31
breq	CAA4

rjmp	CAA2b		;go for another round


CAA4:

swap	r27			;swap data in r27 for processing command DOW information

andi	r27,15		;save 4 lower bits of DOW information
breq	CAA5		;zero for command = every day

lds		r24,$63		;get DOW in r24 for comparison
cp		r27,r24		
breq	CAA5


CAA2b:

cp		r26,r16		;is it at end of bookmarked SRAM memory?
brlo	CAA2

ret					;it has reached end of programmed alarms


CAA5:				;issue relay toggle command 

rcall	ToggleRelay

rjmp	CAA2b		;go for another round


;----------------------------------------------

ButtonStall:

sbis	PIND,0	;skip if switch is up
rjmp	ButtonPush
rjmp	ButtonStall


;-------------------------------------------------------

ButtonPush:		;Takes in dit, dah, holddown, and coded input

push	r20
push	r21
push	r23
push	r28

;r30,r31 (Z register pair contain total value of entry)

clr		r21		;Dit Count
clr		r20		;HoldDown count
clr		r23		;universal counter
clr		r28		;dah count
clr		r0		;clear number of bits in code
clr		r1		;clear lowest code count
clr		r2		;clear next code count
clr		r3		;clear next code count
clr		r4		;clear next code count (clear all code counts)


				;r30,31 has total count

B1:

inc		r23			;universal counter
rcall	Delay40
cpi		r23,60		;delay before first holddown count - 1 1/2 second 
brsh	B3			;branch if - it is a holddown count

sbis	PIND,0		;skip if switch is NOT down
rjmp	B1


B2:

cpi		r23,20		;compare r23 for 20 (1/2 second dah count)
brlo	B2b			;branch if lower (dit count)


B2a:				;it is dah count

inc		r28			;increment dah count

sec					;set carry flag -- roll in dah entry
rol		r1
rol		r2			;rotate data into registers r1-r4
rol		r3
rol		r4

cbi		PortB,0		;on red LED
rcall	Delay40
rcall	Delay40		;delay for longer on of LEDs
rcall	Delay40
rcall	Delay40
sbi		PortB,0		;off LED
clr		r23
inc		r0			;increment number of code digits


B2d:				;no touch/switch up detection

sbis	PIND,0		;skip if switch is up
rjmp	B2e			;jump back - there is a new entry coming in

inc		r23
rcall	Delay40
cpi		r23,70		;has switch been up 70 counts? 40=one second (1.75 seconds)
brsh	B4Decode1
rjmp	B2d


BPEnd1:

rjmp	BPEnd


B2e:

clr		r23
rjmp	B1


B2b:				;Dit count detected

inc		r21			;increment dit count

clc					;clear carry flag -- roll in dit entry
rol		r1
rol		r2			;rotate data into registers r1-r4
rol		r3
rol		r4

cbi		PortB,0		;give short red flash
rcall	Delay40
sbi		PortB,0

inc		r0			;increment number of code digits (r0)
clr		r23			;clear r23
rjmp	B2d


B3:					;Holddown count has been detected

inc		r20			;increment holddown count
cbi		PortB,0
rcall	Delay40
rcall	Delay40
sbi		PortB,0		;flash red LED

clr		r23			;reset r23 to zero count


B4:

inc		r23
rcall	Delay40
sbic	PIND,0		;skip if switch is down
rjmp	B4a			;jump out and look for next entry

cpi		r23,30		;pause between continuous hold down of switch
brlo	B4

rjmp	B3


B4a:

clr		r23
rjmp	B2d			;Check for exit or new entry



B4Decode1:

;This section finalizes.  It combines counts

clr		r30
clr		r31			;ready registers for input

mov		r16,r28		;move dah count in r28 into r16


B4comb1:

tst		r16
breq	B4comb2

adiw 	r30,10		;add value of 10 in r30 to r31 (dah)
dec		r16			
rjmp	B4comb1


B4comb2:

add		r30,r21		;add value in r21 (dit count) to r30
brcs	B4comb2a
rjmp	Bholdown1


B4comb2a:

inc		r31			;carry was set so increment higher byte


Bholdown1:

;Combined dit/dah values now in Z register pair, r30,31

mov		r16,r20		;put hold down count into r16

Bholdown1b:

tst		r16			;test r20(r16), hold down count
breq	BPEnd
adiw	r30,50		;add fifty count to total
dec		r16
rjmp	Bholdown1b

;Complete combined value now in Z register pair


BPEnd:

pop		r28
pop		r23
pop		r21
pop		r20

sbiw	r30,1		;decrement entry by one for Plus One entry system used here

ret


;-------------------------------

;Delay subroutines

;-------------------------

Delay40:

push	r30
push	r31

ldi		r30,97
ldi		r31,87


D40a:

sbiw	r30,1
brne	D40a

pop		r31
pop		r30

ret


;-------------------------------------------

DelayHalf:				;1/2 second delay

push	r16

ldi		r16,20


DHa:

rcall	Delay40
dec		r16
brne	DHa

pop		r16

ret


;--------------------------------------------

DelayOne:

push	r16
ldi		r16,40

rjmp	DHa

 ;-----------------------------------------

 DelayTwo:

 push	r16
 ldi	r16,80
 rjmp	DHa


 ;------------------------------------------


;Escape Delay Routines

;-------------------------------------------


Escape80Delay:		;1/80th second delay.  When touch plate touched, there is quick escape to return

sbis	PIND,0		;skip if touch plate not touched
ret

push	r30
push	r31

ldi		r30,30
ldi		r31,29		;set up delay


E80D2:

sbis	PIND,0		;skip if touch plate not touched
rjmp	E80D1		;exit

sbiw	r30,1		;subtract 1 from r30,31 pair
brne	E80D2


E80D1:

pop		r31
pop		r30

ret


;-------------------------------------------------


Escape40Delay:		;1/40th second delay.

sbis	PIND,0		;skip if touch plate not touched
ret

rcall	Escape80Delay
rcall	Escape80Delay

ret


;---------------------------------------------------

Escape1000thDelay:

sbis	PIND,0		;skip if touch plate not touched
ret

push	r30
push	r31

ldi		r30,81
ldi		r31,2		;set up delay values


E1000th1:

sbis	PIND,0		;skip if touch plate not touched
rjmp	E1000thEnd
sbiw	r30,1		;subtract 1 from z register pair
brne	E1000th1


E1000thEnd:

pop		r31
pop		r30

ret


;-------------------------------------

EscapeTenthDelay:

sbis	PIND,0		;skip if touch plate not touched
ret

rcall	Escape40Delay
rcall	Escape40Delay

sbis	PIND,0		;skip if touch plate not touched
ret

rcall	Escape40Delay
rcall	Escape40Delay

ret


;----------------------------------------------------

EscapeHalfDelay:	;Half Second Delay

sbis	PIND,0		;skip if touch plate not touched
ret

push	r16
ldi		r16,20


EHalf1:

sbis	PIND,0		;skip if touch plate not touched
rjmp	EHalf2

rcall	Escape40Delay
dec		r16
brne	EHalf1


EHalf2:

pop		r16

ret


;-------------------------------------------------------

EscapeOneDelay:		;ONE second delay

sbis	PIND,0		;skip if touch plate not touched
ret


push	r16
ldi		r16,40


EOne1:

sbis	PIND,0		;skip if touch plate not touched
rjmp	Eone2

rcall	Escape40Delay
dec		r16
brne	Eone1


EOne2:

pop		r16

ret


;-------------------------------------------------

FetchEEPROM:		;grabs data in permanent saved EEPROM
					;r16 has address location - r17 returns data
					
cli					;clear interrupts

FEE:

sbic	EECR,EEWE	;Wait for completion of previous write
rjmp	FEE

out		EEAR,r16	;set up address in address register
sbi		EECR,EERE	;start EEPROM read by writing EERE

in		r17,EEDR	;read data from data register - LSB

sei					;on interrupts

ret					;read data now in r17


;-------------------------------------------------

WriteEEPROM:		;write to EEPROM - r16 has memory location, r17 has data

cli					;off interrupts


WEE:

sbic	EECR,EEPE	;Wait until ready for write
rjmp	WEE

out		EEAR,r16	;set up address in address register
out		EEDR,r17	;put contents of r17 into EEPROM data location

sbi		EECR,EEMPE	;write logical one to EEMPE

sbi		EECR,EEPE	;start eeprom write by setting EEPE

sei					;on interrupts

ret


;--------------------------------------------------

OneSecondRFlash:

cbi		PortB,0
rcall	DelayOne
sbi		PortB,0

ret


;------------------------------------------

ToggleRelay:			;Toggles relay on or off. Relay status is recorded in R18,1(2)

sbrc	r18,1			;skip if r18,1(2) is clear -- relay status
rjmp	TRI1

sbr		r18,2			;set r18,1(2) relay status to ON

sbi		PortB,3			;set PortB,3 for pulse of relay ON
rcall	Delay40
cbi		PortB,3			;clear PortB,3 for off of pulse

ret


TRI1:

cbr		r18,2			;set r18,1(2) relay status OFF

sbi		PortB,4			;set PortB,4 to pulse relay OFF
rcall	Delay40
cbi		PortB,4			;clear PortB,4 for off of pulse

ret


;--------------------------------------------

ZeroCommandMemory:	;zero all command storage locations in SRAM

		
clr		r1
ldi		r26,$80	;low start byte for start of storage locations
sts		$7F,r26	;store new bookmark for start of entries at $80

ldi		r16,32	;for zeroing 32 locations 

ZCM:

st		x+,r1	;zero location

dec		r16	
brne	ZCM

ret


;----------------------------------

SetClock:		;sets clock for AM/PM, hour, minutes, and day-of-week
			;  24 hour time entry required

clr		r26		;low byte accumulator
clr		r27		;high byte accumulator

ldi		r16,3		;set up for THREE green flashes
rcall	FlashGLED	;indicate that three entries are needed

rcall	ButtonStall	;Plus one entry system used
sts		$63,r30		;day of week (1=Sunday, 7=Saturday)

rcall	OneGreenFlash 	;one green flashes indicate one entry
rcall	ButtonStall		;enter hours -- 24 hour format -- Plus One entry system is used

mov		r16,r30


SetC1:

adiw	r26,60
dec		r16
brne	SetC1			;Adds 60 minutes for each hour entered
						;  Values put in r26,27 - y register pair

rcall	OneGreenFlash	;indicating one entry needed
rcall	ButtonStall		;Enter minutes -- Plus One entry used 1=0

add		r26,r30
brcc	SetC2

inc		r27


SetC2:

add		r27,r31			;Minutes now added to hour count

rcall	GreenFlashStall	;blinks green LED until button is pushed


SetEnd:

cli					;turn off global interrupts

sts		$61,r26		;store low byte in $61
sts		$62,r27		;store high byte in $62
clr		r15			;zero seconds counter

sei					;turn on global interrupts


SetC3:				;Stalls until touch plate not touched

sbic	PIND,0		;skip if button down
rjmp	SetC4
rcall	Delay40
rjmp	SetC3


SetC4:

ret


;--------------------------------------

;FLASHRLED - flashes Red LED No. of Times in r16 (non-interruptible)

FlashRLED:

tst		r16
breq	FlashRLEDend
						;put value of R16 in R0
cbi		PORTB,0			;Turn on RED LED
rcall	Delay40
sbi		PORTB,0			;Off RED LED
rcall	DelayHalf	
dec		r16
brne	FlashRLED


FlashRLEDend:

ret

;-----------------------

FlashGLED:		;Flash Green LED no. of time in R16 (non-interruptible)

tst		r16
breq	FlashGLEDend

rcall	DelayHalf
cbi		PORTB,1			;Turn on GREEN LED
rcall	Delay40
sbi		PORTB,1			;Off GREEN LED
dec		r16
brne	FlashGLED


FlashGLEDend:

ret

;------------------------------------


OneGreenFlash:

push	r16

ldi		r16,1
rcall	FlashGLED

pop		r16

ret


;------------------------------------

IGFlash:			;Interruptible green flash - R16  contains 
				; number of flashes

tst		r16
breq	IGF1
cbi		PortB,1		;on Green LED
rcall	Escape40Delay
sbi		PortB,1		;off Green LED
rcall	EscapeHalfDelay
dec		r16
rjmp	IGFlash


IGF1:

rcall	EscapeHalfDelay

ret				


;----------------------------


IRFlash:			;Interruptible red flash - R16  contains 
				; number of flashes

tst		r16
breq	IRF1
cbi		PortB,0		;on Green LED
rcall	Escape40Delay
sbi		PortB,0		;off Green LED
rcall	EscapeHalfDelay
dec		r16
rjmp	IRFlash


IRF1:
rcall	EscapeHalfDelay

ret				


;---------------------------

VarWagRG:		;R16 contains number of wags

cbi		PortB,0
rcall	Delay40
sbi		PortB,0
rcall	Delay40
rcall	Delay40
cbi		PortB,1
rcall	Delay40
sbi		PortB,1
rcall	Delay40
rcall	Delay40
dec		r16
brne	VarWagRG

ret

;-----------------------


TwoWags:		;do two wags of the VarWagRG

ldi		r16,2

rcall	VarWagRG

ret


;--------------------------

;NEW code added --- 6/2/2011

;-------------------------------------------

Multiply10X4reg:		;Added 6/28/2011

;Input is R28,29,30,31    Output is for same registers    SRAM  $79-$7C
;r0-r3 = scratch copy of input data for later addition section

mov		r0,r28
mov		r1,r29
mov		r2,r30
mov		r3,r31		;save a copy of input data in r0-3 for later addition

ldi		r16,3		;load r16 for 8X multiplication by shifting data left 3 X


M4a:

lsl		r28
rol		r29
rol		r30
rol		r31			;rotate input data through for 8X multiplication

dec		r16
brne	M4a


;Following section adds r0-r3 two times to copmplete 10X multiplication

ldi		r16,2


M4add:

add		r28,r0
brcc	M4aa

inc		r29


M4aa:

add		r29,r1
brcc	M4b

inc		r30


M4b:

add		r30,r2
brcc	M4c

inc		r31


M4c:

add		r31,r3

dec		r16
brne	M4add		;repeat operation twice

ret					;new 10X multiplied data back in r28-31



;-----------------------------------------

MultiplyBillion4reg:		;data is in r28,29,30,31

rcall Multiply10X4reg	
rcall Multiply10X4reg
rcall Multiply10X4reg


MultiplyMillion4reg:

rcall Multiply10X4reg
rcall Multiply10X4reg
rcall Multiply10X4reg


MultiplyThousand4reg:

rcall Multiply10X4reg
rcall Multiply10X4reg
rcall Multiply10X4reg

ret


;---------------------------------------------

AddClockCorrectionData:	;Accumulator is set up to store added up data (r4-7)

EnterBillions:

ldi		r16,4
rcall	FlashGLED	;indicating four entries are needed

clr		r4
clr		r5
clr		r6
clr		r7			;clear accumulator r4-7


rcall	ButtonTrim
rcall	MultiplyBillion4reg
rcall	AddToAccumulator	;billions data now in accumulator


EnterMillions:

rcall	OneGreenFlash	;indicating one entry due

rcall	ButtonTrim
rcall	MultiplyMillion4reg
rcall	AddToAccumulator	;billions data now in accumulator


EnterThousands:

rcall	OneGreenFlash	;indicating one entry due

rcall	ButtonTrim
rcall	MultiplyThousand4reg
rcall	AddToAccumulator	;billions data now in accumulator


AddHundres:

rcall OneGreenFlash

rcall	ButtonTrim
rcall	AddToAccumulator

ret							;accumulator (r4-r7) now has copiled data


;-----------------------------------------------

ButtonTrim:			;Puts entered data into registers r28,29 and clears r30,31
					;  for later multiplication processing of registers r28-31

rcall	ButtonStall	
					;Plus one entry system used

mov		r16,r30
mov		r17,r31		;save button data in registers to r16,17

clr		r30
clr		r31			;clear two MSB of 4 resisters r28,29,30,31

mov		r28,r16
mov		r29,r17		;move entered touch plate data into r28,29
					;DATA now ready for multiplication in r28-r31
ret


;------------------------------------------------

AddToAccumulator:	;data in r28-31 is added to data in accumulator (r4-7)

add		r4,r28
brcc	ATC1

inc		r5


ATC1:

add		r5,r29
brcc	ATC2

inc		r6


ATC2:

add		r6,r30
brcc	ATC3

inc		r7


ATC3:

add		r7,r31

ret


;-------------------------------------------------

WriteNewAlarmTimes:		;clears old alarm times and writes new SRAM set of alarm times 
						;r28 is bookmark counter  --  r21 24,25,28,29 are scratch

ldi		r28,$80			;start of alarm time section storage area - set book mark to beginning (Y)
sts		$7F,r28			;alarm times start in bookmark location, $7F


AddToAlarmTimes:

lds		r28,$7F			;get book mark value from SRAM $7F

cpi		r28,$AA
brne	NAT1

rjmp	NATend1			;Error exit -- memory is full


NAT1:					;continue and add new data

ldi		r16,3			;give three flashes of RED LED to indicate three entries are needed
rcall	FlashRLED

rcall	ButtonStall		;If entry 10 or more, exit routine - stop continuous entries

cpi		r30,9			;  A dah or more than ten touches will exit series of entries
brlo	NAT1b
rjmp	NATend2			;signal normal exit


NAT1b:

						;(1)0 = every day, (2)1= Sunday, (3)2 = Monday, ..... (8)7 = Saturday

swap	r30				;swap 4 top bit nibbles with 4 bottom bit nibbles to place DOW data in proper place
andi	r30,240			;mask out all but upper four bits
mov		r21,r30			;move r30 into r21 for temporary store

rcall	OneGreenFlash	;green flash shows entry has been made

;---

clr		r24
clr		r25				;clear r24,25 for entry of minutes

rcall	ButtonStall		;enter hours in 24 hour format using Plus One format 1=0


NAT2:

tst		r30
breq	NAT2b

adiw	r24,60			;add 60 minutes to r24,25 (w register pair)
dec		r30
rjmp	NAT2			;go fot another round


NAT2b:

rcall	OneGreenFlash	;to show entry has been made


NAT3:

rcall	ButtonStall		;enter minutes using plus one entry 1=0

add		r24,r30
brcc	NAT2a

inc		r25				;add one for carry to w register pair high byte


;complete hours and minutes alarm time now in r24,25  -- w register pair


NAT2a:

andi	r25,15			;mask out upper 4 bits of r25 for insertion of DOW data
or		r25,r21			;insert DOW data


;Two bytes now complete in r24,25

st		y+,r24			;store low byte with post increment of r28 (y register pair)
st		y+,r25			;store high byte with post increment of r28 (y register pair)

sts		$7F,r28			;store new incremented book mark value


NATend:

rjmp	AddToAlarmTimes	;jump back for another entry


NATend1:				;OVERFLOW - entry not made --- memory full (Long RED LED flash)

rcall	OneSecondRFlash

ret


NATend2:			;normal exit - wag flash

rcall	TwoWags		;(accept wag)

ret


;--------------------------------------------------------------------

AdjustXtlDivider:		;adjusts divider for frequency division and store it in EEPROM
						; Calculated value for this is 159 (Plus one entry system used)

rcall	OneGreenFlash	;to indicate one entry needed

rcall 	ButtonStall		;Plus On system entry 1=0

ldi		r16,54			;set up quarter second interrupts
out		OCR1AH,r16

out		OCR1AL,r30		;loads the compare register A for timer 1 (.25 second interrupts)
	
sts		$67,r30			;store divider in $67 SRAM

ldi		r16,$07			;address in EEPROM form storing of divider
mov		r17,r30			;put data in for storage

rcall	WriteEEPROM		;puts data into $07 of EEPROM

ret


;-----------------------------------------------------------------------	

TransferSRAMtoEEPROM:

;Input Data:  r20=number of bytes to transfer    r16=start of EEPROM data     r30=SRAM z pointer
;  r17 scratch data for transfer to eeprom

ld		r17,z+		;load r17 with data from area pointed to by z register, r30 (with post increment)

rcall	WriteEEPROM		;location=r16, data=r17
inc		r16				;go to next WriteEEPROM write location

dec		r20				;decrement number of bytes to transfer
brne	TransferSRAMtoEEPROM

ret


;----------------------------------------------------------------------

TransferEEPROMtoSRAM:

;Input Data:  r20=number of bytes to transfer     r16=start of EEPROM data     r30=SRAM pointer
;  r17 fetched data from eeprom

rcall	FetchEEPROM		;get data from EEPROM
inc		r16				;got to next Fetch EEPROM location

st		z+,r17		;store fetched data into SRAM area pointed by z register, r30 with post increment

dec		r20				;decrement number of bytes to transfer

brne	TransferEEPROMtoSRAM

ret


;--------------------------------------------------------------

TransferClockCorrectionDataToEEPROM: 	;from SRAM to EEPROM

ldi		r20,16			;sixteen bytes to transfer
ldi		r30,$68			;start of SRAM data location
ldi		r16,$08			;start of EEPROM data location

rcall	TransferSRAMtoEEPROM

ret


;--------------------------------------------------

TransferClockCorrectionFromEEPROM:		;from EEPROM to SRAM

;Transfers clock correction times in EEPROM to SRAM

ldi		r20,16			;sixteen bytes to transfer
ldi		r30,$68			;start of SRAM location
ldi		r16,$08			;start of EEPROM location

rcall	TransferEEPROMtoSRAM

ret


;----------------------------------------------------

TransferAlarmTimesToEEPROM:

ldi		r20,33			;33 bytes to transfer
ldi		r30,$7F			;start of SRAM location
ldi		r16,$20			;start of EEPROM location

rcall	TransferSRAMtoEEPROM


;-----------------------------------------------------

TransferAlarmTimesFromEEPROM:

ldi		r20,33			;33 bytes to transfer
ldi		r30,$7F			;start of SRAM location
ldi		r16,$20			;start of EEPROM location

rcall	TransferEEPROMtoSRAM

ret


;-----------------------------------------------------

ZeroSRAMeeprom:		;Zeros clock corrections and puts 159 in as default for Xtl divider
					;  in both SRAM & EEPROM

clr		r16
ldi		r17,16		;16 locations to zero
ldi		r30,$68		;start of SRAM to be zeroed


PSRAM1:

st		z+,r16		;r30,31, z register has address - z post incremented
dec		r17	
brne	PSRAM1

ldi		r16,159		;This selected for clock to run slightly slow
sts		$67,r16		;159 for low byte divider


ldi		r20,16		;number of locations to transfer
ldi		r16,$08		;location in EEPROM to transfer data to
ldi		r30,$68		;start location in SRAM to transfer data from

rcall	TransferSRAMtoEEPROM

ret


;-----------------------------------------------------------

LoadR28toR31byR26start:

;Loads permanent count from SRAM into r28-31
;  r26, x register pair, is SRAM start pointer
;  Also zero flag is set if result or all zero = 0

ld		r28,x+
ld		r29,x+
ld		r30,x+
ld		r31,x+

ret


;---------------------------------------------------------

ChkR28toR31forZero:

clr		r16
or		r16,r28
or		r16,r29
or		r16,r30
or		r16,r31		;zero flag is set IF result is that of all registers being zero=0

ret


;------------------------------------------------------------

StoreR28toR31byR26start:

st		x+,r28
st		x+,r29
st		x+,r30
st		x+,r31

ret


;------------------------------------------------------------

StoreAccumulatorCount:		;Stores r4-7 accumulator at location pointed to by r26
				;  accumulator, r4-7 has data

st		x+,r4
st		x+,r5
st		x+,r6
st		x+,r7

ret


;-------------------------------------------------------------

StoreCount:			;Stores r28-r31 data into SRAM area pointed to by r26, x register pair

st		x+,r28
st		x+,r29
st		x+,r30
st		x+,r31

ret


;----------------------------------------------------------------

CounterOneCorrectionData:	;four entries to set up interval for clock correction in
							;  counter #1.  This is coarse correction - seconds per
							;  24 hour period with slight slowdown of clock so
							;  counter #2 can do final "fractional" correction.

rcall	AddClockCorrectionData

ldi		r26,$68
rcall	StoreAccumulatorCount			;stores r4-r7 accumulator data in permanent store area of SRAM
							;  r26 has address start of SRAM

ldi		r26,$6C
rcall	StoreAccumulatorCount			;stores r4-7 in variable counter #1 section of SRAM

ldi		r20,8				;r20 contains number of bytes to transfer
ldi		r30,$68				;start of SRAM permanent store area - counter #1
ldi		r16,$08				;r16 has start of EEPROM store area for counter #1

rcall	TransferSRAMtoEEPROM	;data now stored in EEPROM

ret


;--------------------------------------------------------------

CounterTwoCorrectionData:

rcall	AddClockCorrectionData		;four entries to set up interval for clock correction in
									;  counter #2.  This is fine, "fractional" correction
									;  one second is added every so many seconds.
ldi		r26,$70
rcall	StoreAccumulatorCount			;stores r4-7 count in permanent store #2 counter section of SRAM

ldi		r26,$74				
rcall	StoreAccumulatorCount			;stores r4-7 in variable counter #2 section of SRAM

ldi		r20,8				;r20 contains number of bytes to transfer
ldi		r30,$70
ldi		r16,$10				;EEPROM area for permanent store, counter 2

rcall	TransferSRAMtoEEPROM

ret


;---------------------------------------------------------------

Test1:

ldi		R16,20


Test1a:

rcall	EscapeTenthDelay
sbi		PortB,3

rcall	EscapeTenthDelay
cbi		PortB,3

sbi		PortB,4
rcall	EscapeTenthDelay
cbi		PortB,4

dec		r16
brne	Test1a

ret


;--------------------------------------------------------------

Test2:		;Flash out byte - MS first

push	r16
push	r17

rcall 	DelayOne

ldi		r17,8


T2aa:

sbrc	r16,7
rjmp	T2a

rcall	DelayHalf
rcall	OneGreenFlash

rjmp	T2b


T2a:

rcall	DelayHalf
rcall  	OneSecondRFlash


T2b:

lsl		r16
dec		r17
brne	T2aa

pop		r17
pop		r16

rcall	DelayOne
rcall	DelayOne

ret


;--------------------------

QuarterEntries:		;clears all alarm commands and puts a quick, quarter second entry from
					;  00:15 to 23:45.  Allows alarm to stay on one hour.

ldi		r16,1			;give one flash of RED LED to indicate one entry is needed
rcall	FlashRLED

rcall	ButtonStall		;plus one entry
cpi		r30,0
brne	QE2

ret


QE2:

cpi		r30,96
brlo	QE1				;exit if error entry

ret	


QE1:

ldi		r28,$80			;start of alarm time section storage area - set book mark to beginning (Y)
sts		$7F,r28			;alarm times start in bookmark location, $7F

;following AHTM section converts r30 quarter hours to r30,31 minutes

mov		r16,r30			;move number of quarter hours into r16

clr		r30
clr		r31


AHTM1:					;15X number entered in quarter hours

adiw	r30,15			;add 15 minutes to r30,31
dec		r16
brne	AHTM1

;----

st		y+,r30
st		y+,r31			;store away number of minutes after midnight

sts		$7F,r28			;store r28 to book mark location (r28=$84) - two toggle entries

ret


;-----------------------------------------------------------------

